#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <curses.h>
#include <signal.h>
#include <sys/ioctl.h>

#include "hfo.h"
#include "server.h"
#include "curses.h"

void initialize_curses() {
    
    initscr();
    /* start_color(); */
    signal(SIGWINCH, winresize_callback);
    cbreak();
    noecho();
    nodelay(stdscr, TRUE); 
    /* use_default_colors(); */
    clear();
    
}

void winresize_callback(int signum) {

    struct winsize wsiz;

    ioctl(0, TIOCGWINSZ, &wsiz);
    resizeterm(wsiz.ws_row, wsiz.ws_col);
}

void draw_screen(server_t *server) {

    /* char *progressbar; */
    int i, row = 2;
    /* time_t now = time(NULL); */
    clear();
    mvprintw(0, 2, "httpoffer %s", VERSION);
    mvprintw(0, stdscr->_maxx - strlen(CLIENTS_CONNECTED_MSG) - 5,"%s  %d", CLIENTS_CONNECTED_MSG, server->numclients);
     
    for (i = 0; i < server->numclients && server->client[i].request; i++) {
        row = print_client_info(row, &server->client[i]);
    }
    refresh();

}

int print_client_info(int row, client_t *client) {
    
    time_t now = time(NULL);
    char *pbar, *perc, *eta, *sent;
    
    mvprintw(row, 0, "Sending: %s", client->request);
    row++;
    /* Progress bar length:
     * (total x) - (4 chars for percentage) - (2 chars for [ and ]) -
     * (2 spaces) - (13 chars for bytes sent) - (11 chars for kb/s field)
     */ 
    pbar = progress_bar(client->sent, client->total_size, stdscr->_maxx - 30);
    perc = percentage(client->sent, client->total_size);
    sent = readable_size(client->sent);
    mvprintw(row, 0, "%-4.4s[%s] %-13.13s%8.2fK/s", perc, pbar, sent, kilobytes_per_second(client->start_time, now, client->sent));
    free(pbar);
    free(perc);
    free(sent);
    row++;
    mvprintw(row, 0, "To: %s", client->host);
    eta = estimated_time_left(client->start_time, now, client->sent, client->total_size);
    /* Placement of ETA:  (total x) - strlen("ETA: xx:xx:xx") */
    mvprintw(row, stdscr->_maxx - 12, "ETA: %-12.12s", eta);
    free(eta);
    row += 2;
    
    return row;
    
}

char *progress_bar(ssize_t sent, ssize_t total, unsigned int bar_len) {
    
    char *bar;
    double frac;
    int i, marks;
    
    if (bar_len < 8) {
        bar = strdup("N/A");
    }
    else {
        bar = (char *) malloc(bar_len + 1);
        frac = (double) sent / total;
        marks = (int) (frac * bar_len);
        bar[bar_len] = '\0';
        for (i = 0; i < marks; i++) 
            bar[i] = PROGRESSBAR_FILLED_CHAR;

        for (i = marks; i < bar_len; i++)
            bar[i] = PROGRESSBAR_EMPTY_CHAR;
    }
    return bar;
}

float kilobytes_per_second(time_t start, time_t now, ssize_t sent) {
    
    if (now-start != 0)
        return ((float) sent / (now-start)) / 1024.0;
    
    return 0;
}

char *percentage(ssize_t sent, ssize_t total) {
    
    int perc;
    char *output;
    
    if (total != 0) {
        perc = (int) (100.0 * sent / total);
        if (perc == 100) 
            output = strdup("100%");
        else if (perc >= 10) {
            output = (char *) malloc(4);
            sprintf(output, "%d%%", perc);
        }
        else {
            output = (char *) malloc(3);
            sprintf(output, "%d%%", perc);
        }
    }
    else
        output = strdup("100%");
    
    return output;
}

char *estimated_time_left(time_t start, time_t now, ssize_t sent, ssize_t total) {
    
    int eta, eta_h, eta_m, eta_s;
    char *output;
    double bps;
    
    if (sent == 0 || now-start < 1) {
        output = strdup("--:--");
    }
    else {
        bps = ((double) sent / (now-start));
        eta = (int) ((total-sent) / bps);
        
        eta_h = eta / 3600;
        eta %= 3600;
        eta_m = eta / 60;
        eta %= 60;
        eta_s = eta;

        if (eta_h > 99)
            output = strdup(">99 hrs");
        else {
            if (eta_h > 0) {
                /* xx:xx:xx and a \0 */
                output = (char *) malloc(9);
                sprintf(output, "%d:%.2d:%.2d", eta_h, eta_m, eta_s);
            }
            else if (eta_m > 0) {
                /* xx:xx and a \0 */
                output = (char *) malloc(6);
                sprintf(output, "%.2d:%.2d", eta_m, eta_s);
            }
            else {
                /* xx and a \0 */
                output = (char *) malloc(3);
                sprintf(output, "%.2d", eta_s);
            }
        }
    }
    return output;
}
 
char *readable_size(ssize_t size) {

    char *new, *orig = (char *) malloc(FILESIZE_MAX_LEN + 1);
    ssize_t orig_len;
    int counter;
    char *p_new, *p_orig = orig;

    sprintf(orig, "%ld", (long int) size);
    orig_len = strlen(orig);

    /* this will alloc 1 byte extra in some cases */
    new = (char *) malloc(orig_len + orig_len/3 + 1);
    memset(new, 0, orig_len + orig_len/3 + 1);
    p_new = new;

    /* calculate where the first ',' should be */
    counter = orig_len % 3;
    
    if (orig_len > 3 && counter > 0) {
        while (counter > 0 && *p_orig != '\0') {
            *p_new++ = *p_orig++;
            counter--;
        }
        *p_new++ = ',';
    }

    counter = 0;
    while (*p_orig != '\0') {
        *p_new++ = *p_orig++;
        if (counter % 3 == 2 && *p_orig != '\0')
            *p_new++ = ',';
        counter++;

    }
    free(orig);

    return new;

}

