#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>

#define BACKLOG_LENGTH 32  /* size of the queue to listen() */ 
#define MAX_CLIENTS 16
#define BUFSIZE 8192

#define CRLF "\r\n"

#define CODE_200_OK                 "HTTP/1.1 200 OK" CRLF
#define CODE_206_PARTIAL_CONTENT    "HTTP/1.1 206 Partial Content" CRLF
#define CODE_404_NOT_FOUND          "HTTP/1.1 404 Not Found" CRLF
#define CODE_403_FORBIDDEN          "HTTP/1.1 403 Forbidden"
 
#define TEXT_HTML_CONTENT    "Content-Type: text/html" CRLF
#define OCTET_STREAM_CONTENT "Content-Type: application/octet-stream" CRLF

#define CONTENT_RANGE        "Content-Range: bytes %d-%d/%d" CRLF
#define CONTENT_LENGTH       "Content-length: %d" CRLF

#define END_RESPONSE         CRLF

#define NOT_FOUND_BODY       "<html><head><title>404 Not Found</title><style></head><H1>Not Found</H1>The requested URL <code>%s</code> was not found on this server</body></html>\n"
#define FORBIDDEN_BODY       "<html><head><title>403 Forbidden</title><style></head><H1>Forbidden</H1>You don't have permission to access <code>%s</code> on this server</body></html>\n"

#define IS_GET_REQUEST ( client->rbuf[0] == 'G' && client->rbuf[1] == 'E' && client->rbuf[2] == 'T' )
#define RANGE_REQUEST "Range: bytes="

#define INDEX_HEADER "<html><head><title>httpoffer v" VERSION "</title></head><body bgcolor=\"#ffffff\" text=\"#000000\"><pre>Filename                                                      Size<hr noshade align=\"left\" width=\"90%\">\n"
#define INDEX_FOOTER "<hr noshade align=\"left\" width=\"90%\">httpoffer v" VERSION "</pre></body></html>\n"
#define NUM_SPACE_BEFORE_SIZE 20
#define FILESIZE_MAX_LEN 15
#define INDEX_ENTRY_START "<a href=\""
#define INDEX_ENTRY_MIDDLE "\">"
#define INDEX_ENTRY_END "</a>"


enum {
    IS_ROOT,
    IS_REGULAR,
    IS_DIRECTORY,
    IS_INVALID,
    IS_NO_ACCESS
};

enum {
    TEXT_HTML,
    OCTET_STREAM
};

typedef struct {
    int     sock;
    int     fd;
    char    *host;
    char    rbuf[BUFSIZE+1];
    ssize_t bytes_in_buf;
    
    char    *request;
    ssize_t total_size;
    time_t  start_time;
    ssize_t sent; 
    ssize_t file_offset;
    
} client_t;

typedef struct {
    int      sock;
    int      port;
    int      numclients;
    
    char     **filebase;
    int      filenum;

    client_t client[MAX_CLIENTS];
    
} server_t;


server_t *setup_server(char *host, int port); 
int get_connections(server_t *server);
char *resolv_host(struct sockaddr_in *sin);
void initialize_client(client_t *client, char *ip);
void read_requests(server_t *server);
bool_t process_request(client_t *client);
void client_disconnect(server_t *server, int cli_index);
void send_files(server_t *server);
bool_t is_valid_request(server_t *server, int cli_index);
char file_type(char *request);
bool_t send_file(client_t *client);
bool_t send_response(int fd, int status_code);
bool_t send_content_type(int fd, char type);
bool_t send_end_response(int fd);
bool_t send_content_length(int fd, ssize_t len);
bool_t send_partial_content(int fd, ssize_t start, ssize_t stop, ssize_t size);
void send_not_found(int fd, char *item);
void send_forbidden(int fd, char *item);
bool_t parse_client_request(client_t *client);
bool_t send_directory_index(server_t *server, int cli_index, char *directory);
char *base_filename(char *filename);
char *create_file_entry(char *file);
server_t *shutdown_server(server_t *server);

